#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec 12 13:47:48 2017

@author: lessig
"""

import numpy as np
import matplotlib.pyplot as plt

from ray import Ray
from camera import Camera
from plane import Plane
from sphere import Sphere
from object import Object
from intersection import Intersection
from diffuse import Diffuse
from phong import Phong
from mirror import Mirror
from shading import shade


def render( res_x, res_y, scene) :

    cam = Camera( 30, res_x, res_y)

    for ix in range( res_x) :
        for iy in range( res_y) :

            r = cam.generateRay( ix, iy)

            # trace ray through scene
            isect = Intersection() 
            for obj in scene :
                obj.intersect( r, isect)

            if( r.t < 10000.) :
               col = shade( isect.p, isect.n, isect.rho, scene, -r.d, 16)
               cam.image[ix,iy] = col
                
    return cam.image


# define scene
plane = Object( Plane( np.array([0, 1, 0]), np.array([0, -1., 0])),
                Diffuse( np.array([0.8, 0.2, 0.2])))
sphere = Object( Sphere(np.array([0, 0, 7.]), 1.), 
                 Phong( 16, np.array([0.2, 0.7, 0.2])))
light = Object( Plane(np.array([0, 0., -1]), np.array([0, 10., 0])), 
                 Diffuse( np.array([0.8, 0.2, 0.2])),
                 10.0)
scene = [plane, sphere, light]

# render
im = render( 512, 512, scene)

# display result
imdisplay = np.clip( im / (np.max(im)), 0., 1.0)
plt.imshow( np.flip( np.transpose(imdisplay,(1,0,2)), 0))
image = plt.show()
plt.imsave( 'image.png', imdisplay)
